From b95ed769c471cbddcc7d3ee6e0b145c3925e5248 Mon Sep 17 00:00:00 2001
From: Angus Gratton <gus@projectgus.com>
Date: Sat, 28 Jun 2014 09:27:11 +1000
Subject: [PATCH 1/4] stlink-v2: Retry debug register reads on
 STLINK_SWP_DP_WAIT/STLINK_SWD_AP_WAIT status

When working on nrf51 support I was getting STLINK_SWP_DP_WAIT results
intermittently when reading target state. In my case this was
happening as part of armv7m_wait_algorithm(), where the error result
caused the flash load to fail.

The error printed when this happens is:
"Error: jtag status contains invalid mode value", however this is a
common error message when any state read operation fails.

At the moment the *_WAIT results are treated as errors, but they seem
to be very transient and an immediate retry will succeed (I put a max
of 3 retries in the driver just in case, but I've found one is
normally sufficient.)

I'm not familiar enough with SWD/STLINK to know exactly what causes
these, I'm guessing possibly a race condition during the state from
running->halt? Or something like that. The only existing reference I
found to this error was from 2012 regarding STLINK_SWD_AP_WAIT:
http://sourceforge.net/p/openocd/mailman/openocd-devel/thread/50ACF7F6.9000600@op.pl/

This retry fix corrects the issue so I can consistently use the
algorithm routines to execute a flash loader.

Change-Id: I0bcdaa77ca0f28d0852b34f6d987bf8799b0f736
Signed-off-by: Angus Gratton <gus@projectgus.com>
---
 src/jtag/drivers/stlink_usb.c | 27 ++++++++++++++++++++++-----
 1 file changed, 22 insertions(+), 5 deletions(-)

diff --git a/src/jtag/drivers/stlink_usb.c b/src/jtag/drivers/stlink_usb.c
index b4dc0de..e89b575 100644
--- a/src/jtag/drivers/stlink_usb.c
+++ b/src/jtag/drivers/stlink_usb.c
@@ -760,6 +760,8 @@ static int stlink_usb_idcode(void *handle, uint32_t *idcode)
 static int stlink_usb_v2_read_debug_reg(void *handle, uint32_t addr, uint32_t *val)
 {
 	struct stlink_usb_handle_s *h = handle;
+	int retries = 3;
+	int status;
 	int res;
 
 	assert(handle != NULL);
@@ -771,14 +773,29 @@ static int stlink_usb_v2_read_debug_reg(void *handle, uint32_t addr, uint32_t *v
 	h_u32_to_le(h->cmdbuf+h->cmdidx, addr);
 	h->cmdidx += 4;
 
-	res = stlink_usb_xfer(handle, h->databuf, 8);
+	while (retries > 0) {
+		res = stlink_usb_xfer(handle, h->databuf, 8);
 
-	if (res != ERROR_OK)
-		return res;
+		if (res != ERROR_OK)
+			return res;
 
-	*val = le_to_h_u32(h->databuf + 4);
+		*val = le_to_h_u32(h->databuf + 4);
+		status = h->databuf[0];
+
+		if (status == STLINK_DEBUG_ERR_OK)
+			return ERROR_OK;
+
+		if (status != STLINK_SWD_AP_WAIT && status != STLINK_SWD_DP_WAIT) {
+			LOG_DEBUG("stlink read_debug_reg failed with status 0x%0x", status);
+			return ERROR_FAIL;
+		}
+
+		LOG_DEBUG("read_debug_reg returned transient wait status 0x%x", status);
+		retries--;
+	}
+	LOG_DEBUG("read_debug_reg failing after consistent wait status result");
+	return ERROR_FAIL;
 
-	return h->databuf[0] == STLINK_DEBUG_ERR_OK ? ERROR_OK : ERROR_FAIL;
 }
 
 static int stlink_usb_write_debug_reg(void *handle, uint32_t addr, uint32_t val)
-- 
2.0.0

